#include <UnitTest++/UnitTestPP.h>
#include <cctype>
#include <string>
#include <vector>

using namespace std;

int first_negative(const vector<int>& v)
{
    for (vector<int>::const_iterator i = begin(v);
         i != end(v);
         ++i)
    {
        if (*i < 0) return *i;
    }

    return 0;
}

TEST(First_negative)
{
    CHECK_EQUAL(-3, first_negative({2, -3, 5, -9}));
    CHECK_EQUAL(0, first_negative({2, 3, 5, 9}));
}

void in_place_abs(vector<int>& v)
{
    for (vector<int>::iterator i = begin(v); i != end(v); ++i) {
        if (*i < 0) *i = -*i;
    }
}

TEST(In_place_abs)
{
    vector<int> v{2, -3, 5, -9};
    vector<int> w{2,  3, 5,  9};

    in_place_abs(v);
    CHECK(v == w);
}

vector<int> reverse(const vector<int>& v)
{
    vector<int> result;

    for (auto i = rbegin(v); i != rend(v); ++i)
        result.push_back(*i);

    return result;
}

TEST(Reverse)
{
    vector<int> v{2, 3, 4};
    vector<int> w{4, 3, 2};

    CHECK(w == reverse(v));
}

bool is_palindrome_basic(const string& str)
{
    string::const_iterator fwd = begin(str);
    string::const_iterator fwd_end = end(str);
    string::const_reverse_iterator rev = rbegin(str);

    while (fwd != fwd_end)
        if (*fwd++ != *rev++)
            return false;

    return true;
}

TEST(Is_palindrome_basic)
{
    CHECK(is_palindrome_basic("racecar"));
    CHECK(!is_palindrome_basic("race car"));
}

bool is_palindrome(const string& str)
{
    auto fwd     = begin(str);
    auto fwd_end = end(str);

    auto rev     = rbegin(str);
    auto rev_end = rend(str);

    for (;;) {
        while (fwd != fwd_end && !isalnum(*fwd)) ++fwd;
        while (rev != rev_end && !isalnum(*rev)) ++rev;

        if (fwd == fwd_end) return true;
        if (tolower(*fwd) != tolower(*rev)) return false;

        ++fwd;
        ++rev;
    }
}

TEST(Is_palindrome)
{
    CHECK(is_palindrome("racecar"));
    CHECK(is_palindrome("race car"));
    CHECK(is_palindrome("Race car"));
    CHECK(is_palindrome("Race car!"));

    CHECK(!is_palindrome("Racing car"));
}
